import os
import logging
from unittest.mock import MagicMock, patch, mock_open

from django.test import TestCase
from django.conf import settings

import cra_helper
from cra_helper import asset_manifest

# Quiet down logging
logging.disable(logging.CRITICAL)

server_url = 'http://foo.bar'


class TestGenerateManifest(TestCase):
    def test_returns_dict(self):
        self.assertIsInstance(asset_manifest.generate_manifest(True, '', ''), dict)

    def test_returns_bundle_url_if_cra_is_running(self):
        self.assertEqual(asset_manifest.generate_manifest(True, server_url, ''), {
            'bundle_js': server_url
        })

    def test_returns_main_paths_if_cra_is_not_running(self):
        open_mock = mock_open(
            read_data='''{
                "main.js": "static/js/main.1234.js",
                "main.css": "static/css/main.1234.css"
            }'''
        )
        with patch('builtins.open', open_mock):
            self.assertEqual(asset_manifest.generate_manifest(False, server_url, '.'), {
                'main_js': 'js/main.1234.js',
                'main_css': 'css/main.1234.css',
            })

    def test_checks_static_root_when_manifest_missing_from_cra_build_dir(self):
        # Pretend this folder is STATIC_ROOT to emulate falling back and accessing the
        # collected asset-manifest.json
        _here_path = os.path.dirname(os.path.realpath(__file__))
        setattr(settings, 'STATIC_ROOT', _here_path)
        self.assertEqual(asset_manifest.generate_manifest(False, server_url, 'not_a_real_dir'), {
            'main_js': 'js/main.1234.js',
            'main_css': 'css/main.1234.css',
        })

    def test_returns_empty_dict_if_file_not_found(self):
        open_mock = MagicMock(side_effect=Exception)
        with patch('builtins.open', open_mock):
            self.assertEqual(asset_manifest.generate_manifest(False, server_url, '.'), {})



import os
import logging
import json

from django.conf import settings

logger = logging.getLogger(__name__)

_asset_filename = 'asset-manifest.json'


def generate_manifest(is_server_live: bool, bundle_path: str, app_dir: str) -> dict:
    # Prepare references to various files frontend
    if is_server_live:
        return {
            'bundle_js': bundle_path,
        }
    else:
        _manifest = {}
        build_dir = os.path.join(app_dir, 'build')

        # Add the CRA static directory to STATICFILES_DIRS so collectstatic can grab files in there
        static_dir = os.path.join(build_dir, 'static')
        settings.STATICFILES_DIRS += [static_dir]

        # CRA generates a JSON file that maps typical filenames to their hashed filenames
        manifest_path = os.path.join(build_dir, _asset_filename)

        # Try to load the JSON manifest from the React build directory first
        try:
            with open(manifest_path) as data_file:
                logger.info('found manifest in React build files')
                data = json.load(data_file)
        except Exception as e:
            # If that doesn't work, try to load it from the Django project's static files directory
            try:
                static_manifest_path = os.path.join(settings.STATIC_ROOT, _asset_filename)
                with open(static_manifest_path) as data_file:
                    logger.info('found manifest in static files')
                    data = json.load(data_file)
            except Exception as e:
                logger.error('can\'t load static asset manifest: {}'.format(e))
                return {}

        # Generate relative paths to our bundled assets
        for filename, path in data.items():
            asset_key = filename.replace('.', '_')
            asset_key = asset_key.replace('/', '_')

            _manifest[asset_key] = os.path.relpath(path, 'static/')

        return _manifest



import os

from django.conf import settings

from cra_helper.server_check import is_server_live
from cra_helper.asset_manifest import generate_manifest

# Variables to help the template determine if it should try loading assets from the CRA live server
_CRA_DEFAULT_PORT = 3000
_port = _CRA_DEFAULT_PORT

# Allow the user to specify the port create-react-app is running on
if hasattr(settings, 'CRA_PORT') and type(settings.CRA_PORT) is int:
    _port = settings.CRA_PORT

# The URL the create-react-app liveserver is accessible at
CRA_URL = 'http://localhost:{}'.format(_port)

if hasattr(settings, 'CRA_APP_NAME'):
    CRA_APP_NAME = settings.CRA_APP_NAME
else:
    CRA_APP_NAME = 'react'

# The ability to access this file means the create-react-app liveserver is running
CRA_BUNDLE_PATH = '{}/static/js/bundle.js'.format(CRA_URL)

# Check if Create-React-App live server is up and running
CRA_LIVE = is_server_live(CRA_BUNDLE_PATH)

# The path to the CRA project directory, relative to the Django project's base directory
CRA_FS_APP_DIR = os.path.join(settings.BASE_DIR, CRA_APP_NAME)

# A list of entries in CRA's build bundle's
STATIC_ASSET_MANIFEST = generate_manifest(CRA_LIVE, CRA_BUNDLE_PATH, CRA_FS_APP_DIR)



import logging
from urllib import request, error as url_error

from django.conf import settings


def is_server_live(server_path: str) -> bool:
    # Ignore the server check if we're in production
    if settings.DEBUG:
        try:
            resp = request.urlopen(server_path)
            if resp.status == 200:
                logging.info('CRA liveserver is running')
                return True
            else:
                logging.warning('CRA liveserver is up but not serving bundle.js')
                return False
        except url_error.URLError as err:
            logging.warning('CRA liveserver is not running')
            return False
    else:
        return False



from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

setup(
    name = 'django-cra-helper',
    version = '1.0.1',
    description = 'The missing piece of the Django + React puzzle',
    long_description = 'A Django app that allows you to easily incorporate create-react-app code in your Django project',
    url = 'https://github.com/MasterKale/django-cra-helper',
    author = 'Matthew Miller',
    author_email = 'kale@iammiller.com',
    license = 'MIT',
    classifiers = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
    keywords = 'django react create-react-app integrate',
    packages = find_packages(exclude=['contrib', 'docs', 'tests']),
    download_url = 'https://github.com/MasterKale/django-cra-helper/archive/1.0.tar.gz',
    install_requires = [
        'bleach>=2'
    ],
)
